package com.yida.data.auth.service;

import com.yida.data.auth.entity.BindUser;
import com.yida.data.common.core.entity.FebsResponse;
import com.yida.data.common.core.entity.system.UserConnection;
import com.yida.data.common.core.exception.FebsException;
import me.zhyd.oauth.model.AuthCallback;
import me.zhyd.oauth.model.AuthUser;
import me.zhyd.oauth.request.AuthRequest;
import org.springframework.security.oauth2.common.OAuth2AccessToken;

import java.util.List;
import java.util.Map;

/**
 * @author MrBird
 */
public interface SocialLoginService {

    /**
     * 解析第三方登录请求
     *
     * @param oauthType 第三方平台类型
     * @return AuthRequest
     * @throws FebsException 异常
     */
    AuthRequest renderAuth(String oauthType) throws FebsException;

    /**
     * 处理第三方登录（绑定页面）
     *
     * @param oauthType 第三方平台类型
     * @param callback  回调
     * @return FebsResponse
     * @throws FebsException 异常
     */
    FebsResponse resolveBind(String oauthType, AuthCallback callback) throws FebsException;

    /**
     * 处理第三方登录（登录页面）
     *
     * @param oauthType 第三方平台类型
     * @param callback  回调
     * @return FebsResponse
     * @throws FebsException 异常
     */
    FebsResponse resolveLogin(String oauthType, AuthCallback callback) throws FebsException;

    /**
     * 绑定并登录
     *
     * @param bindUser 绑定用户
     * @param authUser 第三方平台对象
     * @return OAuth2AccessToken 令牌对象
     * @throws FebsException 异常
     */
    OAuth2AccessToken bindLogin(BindUser bindUser, AuthUser authUser) throws FebsException;

    /**
     * 注册并登录
     *
     * @param registUser 注册用户
     * @param authUser   第三方平台对象
     * @return OAuth2AccessToken 令牌对象
     * @throws FebsException 异常
     */
    OAuth2AccessToken signLogin(BindUser registUser, AuthUser authUser) throws FebsException;

    /**
     * 绑定
     *
     * @param bindUser 绑定对象
     * @param authUser 第三方平台对象
     * @throws FebsException 异常
     */
    void bind(BindUser bindUser, AuthUser authUser) throws FebsException;

    /**
     * 解绑
     *
     * @param bindUser  绑定对象
     * @param oauthType 第三方平台对象
     * @throws FebsException 异常
     */
    void unbind(BindUser bindUser, String oauthType) throws FebsException;

    /**
     * 根据用户名获取绑定关系
     *
     * @param username 用户名
     * @return 绑定关系集合
     */
    List<UserConnection> findUserConnections(String username);

    /**
     * 企业微信用户登录/注册登录
     */
    Map<String, Object> qywxLogin(String code, String state, String corpId, Integer type);

    /**
     * app用户登录
     */
    Map<String, Object> appLogin(String areaId, Long schoolId, String mobile, String userId, Integer type);

    Map<String, Object> wxpublicLogin(String code, String appId, Integer type, Long schoolId, String mobile);

    Map<String, Object> h5Login(Long schoolId, String mobile, Integer type);

    /**
     * 检查openId状态
     *
     * @param code   微信公众号授权code
     * @param deptId 学校id
     * @return java.util.Map<java.lang.String, java.lang.Object>
     * @author ZYJ
     * @date 2023/6/2 10:31
     */
    Map<String, Object> checkOpenId(String code, Long deptId);

    /**
     * 迎新功能微信公众号登录
     *
     * @param openId 微信openId
     * @param deptId 学校id
     * @return java.util.Map<java.lang.String, java.lang.Object>
     * @author ZYJ
     * @date 2023/5/18 11:09
     */
    Map<String, Object> wxPublicLogin(String openId, Long deptId);
}
