package com.yida.data.common.core.entity.atlas;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.yida.data.common.core.enums.EnableStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * 图集管理
 * 
 * @author ZYJ
 * @date 2021/9/1
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@TableName("core_atlas_management")
public class CoreAtlasManagement implements Serializable {

    private static final long serialVersionUID = -8791305794144044811L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 学校id
     */
    private Long schoolId;

    /**
     * 图集名称
     */
    private String atlasName;

    /**
     * 图集封面图片
     */
    private String atlasCoverImgUrl;

    /**
     * 图集背景图片
     */
    private String atlasBackgroundImgUrl;

    /**
     * 图集简介
     */
    private String atlasIntroduction;

    /**
     * 是否启用：0未启用，1已启用 {@link EnableStatusEnum}
     */
    private Integer enableStatus;

    /**
     * 访问数
     */
    private Integer accessNum;

    /**
     * 点赞数
     */
    private Integer likeNum;

    /**
     * 转发数
     */
    private Integer forwardNum;

    /**
     * 评论数
     */
    private Integer commentNum;

    /**
     * 创建人id
     */
    @TableField(value = "create_id", fill = FieldFill.INSERT)
    private Long createId;

    /**
     * 创建时间
     */
    @TableField(value = "create_date", fill = FieldFill.INSERT)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createDate;

    /**
     * 最后修改人id
     */
    @TableField(value = "update_id", fill = FieldFill.UPDATE)
    private Long updateId;

    /**
     * 最后修改时间
     */
    @TableField(value = "update_date", fill = FieldFill.UPDATE)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime updateDate;

    /**
     * 删除标识. 0: 未删除, 1: 已删除
     */
    @TableLogic(value = "0", delval = "1")
    private Integer delFlag;
}