package com.yida.data.common.core.entity.constant;

/**
 * 企业微信服务商常量
 *
 * @author ZYJ
 * @date 2022/11/3
 */
public interface QywxServiceProviderConstant {

    //------------------企业微信服务商接口地址--------------------
    /**
     * 获取服务商凭证
     */
    String GET_PROVIDER_TOKEN_URL = "https://qyapi.weixin.qq.com/cgi-bin/service/get_provider_token";

    /**
     * 获取第三方应用凭证（suite_access_token）
     */
    String GET_SUITE_TOKEN_URL = "https://qyapi.weixin.qq.com/cgi-bin/service/get_suite_token";

    /**
     * 获取企业凭证
     */
    String GET_CORP_TOKEN_URL = "https://qyapi.weixin.qq.com/cgi-bin/service/get_corp_token?suite_access_token=";

    /**
     * 获取企业永久授权码
     */
    String GET_PERMANENT_CODE_URL = "https://qyapi.weixin.qq.com/cgi-bin/service/get_permanent_code?suite_access_token=";

    /**
     * 获取企业授权信息
     */
    String GET_AUTH_INFO_URL = "https://qyapi.weixin.qq.com/cgi-bin/service/get_auth_info?suite_access_token=";

    /**
     * corpid转换
     */
    String CONVERT_CORP_ID_URL = "https://qyapi.weixin.qq.com/cgi-bin/service/corpid_to_opencorpid?provider_access_token=";

    /**
     * userid的转换
     * 将企业主体下的明文userid转换为服务商主体下的密文userid
     */
    String CONVERT_USER_ID_TO_SECRET_URL = "https://qyapi.weixin.qq.com/cgi-bin/batch/userid_to_openuserid?access_token=";

    /**
     * userid的转换
     * 将代开发应用或第三方应用获取的密文open_userid转换为明文userid
     */
    String CONVERT_USER_ID_TO_PLAIN_URL = "https://qyapi.weixin.qq.com/cgi-bin/batch/openuserid_to_userid?access_token=";

    /**
     * 手机号获取userid
     */
    String GET_USER_ID_BY_MOBILE_URL = "https://qyapi.weixin.qq.com/cgi-bin/user/getuserid?access_token=";

    //------------------企业微信服务商redis key--------------------
    /**
     * 代开发应用模板ticket
     */
    String SERVICE_SUITE_TICKET = "service:suite:ticket:";

    /**
     * 企业微信 corp_token key前缀
     */
    String CORP_TOKEN = "corp:token.";

    /**
     * 企业微信服务商 provider_token key前缀
     */
    String PROVIDER_TOKEN = "provider:token.";

    /**
     * 企业微信服务商 suite_access_token key前缀
     */
    String SUITE_ACCESS_TOKEN = "suite:access.token.";

    /**
     * 服务商对应学校加密corpId数据(加密值为key)
     */
    String PROVIDER_SCHOOL_CORP_DATA = "provider:school:corp:data";

    /**
     * 服务商对应学校加密corpId数据(原始值为key)
     */
    String PROVIDER_SCHOOL_CORP_ORIGINAL_DATA = "provider:school:corp:original:data";

    //------------------企业微信服务商InfoType--------------------
    /**
     * 授权安装待开发应用回调
     */
    String SERVICE_CREATE_AUTH = "create_auth";

    /**
     * 客户删除应用
     */
    String SERVICE_CANCEL_AUTH = "cancel_auth";

    /**
     * 代开发应用重置secret(即permanent_code)
     */
    String RESET_PERMANENT_CODE = "reset_permanent_code";

    /**
     * 待开发应用模板suite_ticket
     */
    String TEMPLATE_SUITE_TICKET = "suite_ticket";
}
