package com.yida.data.common.core.entity.health;

import java.util.Date;
import java.math.BigDecimal;

import lombok.Data;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;

import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.format.annotation.DateTimeFormat;

import java.time.LocalDateTime;
import java.time.LocalDate;
import java.time.LocalTime;

import io.swagger.annotations.ApiModelProperty;

/**
 * 学生单项成绩记录表 Entity
 *
 * @author wjm
 * @date 2021-09-14 17:16:24
 */
@Data
@TableName("edu_health_student_project_test_result")
public class EduStudentProjectTestResult {

    /**
     * 主键
     */
    @ApiModelProperty(value = "主键")
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @TableField(value = "create_date", fill = FieldFill.INSERT)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createDate;

    /**
     * 创建人ID
     */
    @ApiModelProperty(value = "创建人ID")
    @TableField(value = "create_id", fill = FieldFill.INSERT)
    private Long createId;

    /**
     * 创建人名字
     */
    @ApiModelProperty(value = "创建人名字")
    @TableField(value = "create_name", fill = FieldFill.INSERT)
    private String createName;

    /**
     * 最后修改时间
     */
    @ApiModelProperty(value = "最后修改时间")
    @TableField(value = "update_date", fill = FieldFill.UPDATE)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime updateDate;

    /**
     * 最后修改人ID
     */
    @ApiModelProperty(value = "最后修改人ID")
    @TableField(value = "update_id", fill = FieldFill.UPDATE)
    private Long updateId;

    /**
     * 最后修改人姓名
     */
    @ApiModelProperty(value = "最后修改人姓名")
    @TableField(value = "update_name", fill = FieldFill.UPDATE)
    private String updateName;

    /**
     * 是否删除，1删除0未删除	默认0
     */
    @ApiModelProperty(value = "是否删除，1删除0未删除	默认0")
    @TableField("del_flag")
    @TableLogic(value = "0", delval = "1")
    private String delFlag;

    /**
     * 学生主键id
     */
    @ApiModelProperty(value = "学生主键id")
    @TableField("student_id")
    private Long studentId;

    /**
     * 项目主键id
     */
    @ApiModelProperty(value = "项目主键id")
    @TableField("project_id")
    private Long projectId;

    /**
     * 项目名称
     */
    @ApiModelProperty(value = "项目名称")
    @TableField("project_name")
    private String projectName;

    /**
     * 单项项目成绩（同时也是额外加分的项目的成绩，BMI值）
     */
    @ApiModelProperty(value = "单项项目成绩（同时也是额外加分的项目的成绩，BMI值）")
    @TableField("project_result")
    private String projectResult;
    /**
     * 单项项目得分
     */
    @ApiModelProperty(value = "单项项目得分")
    @TableField("project_score")
    private BigDecimal projectScore;
    /**
     * 单项项目等级
     */
    @ApiModelProperty(value = "单项项目等级")
    @TableField("project_rank")
    private String projectRank;

    /**
     * 项目附加分数
     */
    @ApiModelProperty(value = "项目附加分数")
    @TableField("project_extra_score")
    private String projectExtraScore;

    /**
     * 附加分和单项得分的总和
     */
    @ApiModelProperty(value = "附加分和单项得分的总和")
    @TableField("total_project_score")
    private String totalProjectScore;

    /**
     * 学生学年成绩的主键id
     */
    @ApiModelProperty(value = "学生学年成绩的主键id")
    @TableField("grade_result_id")
    private Long gradeResultId;

    /**
     * 学生学号
     */
    @ApiModelProperty(value = "学生学号")
    @TableField("student_num")
    private String studentNum;

    /**
     * 所在学校ID
     */
    @ApiModelProperty(value = "所在学校ID")
    @TableField("school_id")
    private Long schoolId;

    /**
     * 学校名称
     */
    @ApiModelProperty(value = "学校名称")
    @TableField("school_name")
    private String schoolName;

    /**
     * 所在年级ID
     */
    @ApiModelProperty(value = "所在年级ID")
    @TableField("grade_id")
    private Long gradeId;

    /**
     * 年级名称
     */
    @ApiModelProperty(value = "年级名称")
    @TableField("grade_name")
    private String gradeName;

    /**
     * 班级ID
     */
    @ApiModelProperty(value = "班级ID")
    @TableField("class_id")
    private Long classId;

    /**
     * 班级名称
     */
    @ApiModelProperty(value = "班级名称")
    @TableField("class_name")
    private String className;

    /**
     * 身高（如果为BMI才会有身高和体重）
     */
    @ApiModelProperty(value = "身高（如果为BMI才会有身高和体重）")
    @TableField("height")
    private BigDecimal height;

    /**
     * 体重
     */
    @ApiModelProperty(value = "体重")
    @TableField("weight")
    private BigDecimal weight;

    /**
     * 来源类型0为学校录入的信息，1为家长自测录入的信息
     */
    @ApiModelProperty(value = "来源类型0为学校录入的信息，1为家长自测录入的信息")
    @TableField("source_type")
    private Integer sourceType;

    /**
     * 当前填报的年级id
     */
    @ApiModelProperty(value = "当前填报的年级id")
    @TableField("record_grade_id")
    private Long recordGradeId;

}
