package com.yida.data.common.core.entity.program;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * 小程序表 Entity
 *
 * @author ZYJ
 * @date 2021-12-10 13:44:31
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@TableName("edu_small_program")
public class EduSmallProgram implements Serializable {

    private static final long serialVersionUID = -4520043606052720431L;

    /**
     * 主键
     */
    @ApiModelProperty(value = "主键")
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 创建时间
     */
    @ApiModelProperty(value = "创建时间")
    @TableField(value = "create_date", fill = FieldFill.INSERT)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createDate;

    /**
     * 创建人ID
     */
    @ApiModelProperty(value = "创建人ID")
    @TableField(value = "create_id", fill = FieldFill.INSERT)
    private Long createId;

    /**
     * 最后修改时间
     */
    @ApiModelProperty(value = "最后修改时间")
    @TableField(value = "update_date", fill = FieldFill.UPDATE)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime updateDate;

    /**
     * 最后修改人ID
     */
    @ApiModelProperty(value = "最后修改人ID")
    @TableField(value = "update_id", fill = FieldFill.UPDATE)
    private Long updateId;

    /**
     * 修改人名字
     */
    @ApiModelProperty(value = "修改人名字")
    @TableField(value = "update_name", fill = FieldFill.INSERT)
    private String updateName;

    /**
     * 是否删除，1删除0未删除.默认0
     */
    @ApiModelProperty(value = "是否删除，1删除0未删除.默认0")
    @TableField("del_flag")
    @TableLogic(value = "0", delval = "1")
    private Integer delFlag;

    /**
     * 学校或教育局id
     */
    @ApiModelProperty(value = "学校或教育局id")
    @TableField("dept_id")
    private Long deptId;

    /**
     * 学校或教育局名字
     */
    @ApiModelProperty(value = "学校或教育局名字")
    @TableField("dept_name")
    private String deptName;

    /**
     * 小程序名称
     */
    @ApiModelProperty(value = "小程序名称")
    @TableField("small_program_name")
    private String smallProgramName;

    /**
     * 小程序类型id
     */
    @ApiModelProperty(value = "小程序类型id")
    @TableField("type_id")
    private String typeId;

    /**
     * 小程序类型名称
     */
    @ApiModelProperty(value = "小程序类型名称")
    @TableField("type_name")
    private String typeName;

    /**
     * 链接地址
     */
    @ApiModelProperty(value = "链接地址")
    @TableField("link_url")
    private String linkUrl;

    /**
     * 小程序logo图片
     */
    @ApiModelProperty(value = "小程序logo图片")
    @TableField("cover_logo_img")
    private String coverLogoImg;

    /**
     * 小程序logo图片下方名称展示类型
     */
    @ApiModelProperty(value = "为不展示，1为展示")
    @TableField("logo_type")
    private Integer logoType;

    /**
     * 关键词组, 多个用英文逗号隔开
     */
    @ApiModelProperty(value = "关键词组, 多个用英文逗号隔开")
    private String keyWords;

    /**
     * 分享标题
     */
    @ApiModelProperty(value = "分享标题")
    @TableField("share_title")
    private String shareTitle;

    /**
     * 分享地址
     */
    @ApiModelProperty(value = "分享地址")
    @TableField("share_url")
    private String shareUrl;

    /**
     * 是否显示分享（0为不显示，1为显示）
     */
    @ApiModelProperty(value = "是否显示分享（0为不显示，1为显示）")
    @TableField("is_share")
    private Integer isShare;

    /**
     * 分享图片
     */
    @ApiModelProperty(value = "分享图片")
    @TableField("share_img")
    private String shareImg;

    /**
     * 排序数字（按照排序数字进行排序，相同则按照最新发布时间进行排序）
     */
    @ApiModelProperty(value = "排序数字（按照排序数字进行排序，相同则按照最新发布时间进行排序）")
    @TableField("sort_num")
    private Integer sortNum;

    /**
     * app小程序id
     */
    @ApiModelProperty(value = "app小程序id")
    @TableField("app_small_program_id")
    private String appSmallProgramId;

    /**
     * 区县id
     */
    @TableField(exist = false)
    private String areaId;

    /**
     * 0不上架展示，1上架展示默认0
     */
    @ApiModelProperty(value = "0不上架展示，1上架展示默认0")
    @TableField("show_type")
    private Integer showType;

    /**
     * 1栏目顶部排列；2浮动图标右上侧；3浮动图标右中侧；4浮动图标右下侧；5栏目所属文章底部
     */
    @ApiModelProperty(value = "1栏目顶部排列；2浮动图标右上侧；3浮动图标右中侧；4浮动图标右下侧；5栏目所属文章底部")
    @TableField("show_position")
    private Integer showPosition;
}