package com.yida.data.common.core.entity.school;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.yida.data.common.core.enums.ExpiredStatusEnum;
import com.yida.data.common.core.enums.ReportStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.io.Serializable;
import java.time.LocalDateTime;

/**
 * 学校智慧迎新-学生报到信息表
 * 学生信息验证表
 *
 * @author ZYJ
 * @date 2021/8/20
 */
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@TableName("edu_school_student_information")
public class EduSchoolStudentInformation implements Serializable {

    private static final long serialVersionUID = -9193107209586831017L;

    /**
     * 主键id
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 创建人id
     */
    @TableField(value = "create_id", fill = FieldFill.INSERT)
    private Long createId;

    /**
     * 创建人时间
     */
    @TableField(value = "create_date", fill = FieldFill.INSERT)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime createDate;

    /**
     * 最后修改人id
     */
    @TableField(value = "update_id", fill = FieldFill.UPDATE)
    private Long updateId;

    /**
     * 最后修改时间
     */
    @TableField(value = "update_date", fill = FieldFill.UPDATE)
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    private LocalDateTime updateDate;

    /**
     * 删除标识. 0: 未删除, 1: 已删除
     */
    @TableField("del_flag")
    @TableLogic(value = "0", delval = "1")
    private Integer delFlag;

    /**
     * 学生姓名
     */
    @TableField("student_name")
    private String studentName;

    /**
     * 身份证号码
     */
    @TableField("student_id_card")
    private String studentIdCard;

    /**
     * 学号
     */
    @TableField("student_number")
    private String studentNumber;

    /**
     * 家长关系
     */
    private String parentTypeName;

    /**
     * 家长电话
     */
    private String parentTelephone;

    /**
     * 学校id
     */
    private Long schoolId;

    /**
     * 学校名称
     */
    private String schoolName;

    /**
     * 校区id
     */
    private Long campusId;

    /**
     * 校区名称
     */
    private String campusName;

    /**
     * 学段id
     */
    private Long sectionId;

    /**
     * 学段名称
     */
    private String sectionName;

    /**
     * 年级id
     */
    private Long gradeId;

    /**
     * 年级名称
     */
    private String gradeName;

    /**
     * 班级id
     */
    private Long classId;

    /**
     * 班级名称
     */
    private String className;

    /**
     * 报到状态. 0: 未报到, 1: 已报到 {@link ReportStatusEnum}
     */
    private Integer reportStatus;

    /**
     * 过期状态. 0: 未过期, 1: 已过期 {@link ExpiredStatusEnum}
     */
    private Integer expiredStatus;
}
