package com.yida.data.mall.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.yida.data.common.core.entity.mall.EduBillOrder;

import java.math.BigDecimal;
import java.util.Map;

/**
 * 账单订单表 Service接口
 *
 * @author ZYJ
 * @date 2022-03-31 14:49:59
 */
public interface EduBillOrderService extends IService<EduBillOrder> {

    /**
     * 查询已支付金额
     *
     * @param billId    账单id
     * @param studentId 学生id
     * @return java.math.BigDecimal
     * @author ZYJ
     * @date 2022/3/31 15:35
     */
    BigDecimal sumPayOrderPrice(Long billId, Long studentId);

    /**
     * 更新超时账单订单状态
     * 未支付状态
     *
     * @author ZYJ
     * @date 2022/4/14 10:47
     */
    void updateBillOrderStatus();

    /**
     * 更新退款订单状态
     *
     * @author ZYJ
     * @date 2022/4/18 11:16
     */
    void updateRefundStatus();

    /**
     * 查询账单缴费人数
     *
     * @param billId 账单id
     * @return java.lang.Long
     * @author ZYJ
     * @date 2022/4/18 16:29
     */
    Long sumPayStudentNumber(Long billId);

    /**
     * 银联支付通知结果回调
     *
     * @param map 回调值
     * @return java.lang.String
     * @author ZYJ
     * @date 2022/6/27 15:03
     */
    String unionPayNotify(Map<String, Object> map);

    /**
     * 部门缴费总金额
     *
     * @param billId 账单id
     * @param deptId 家校部门id
     * @param type   家校部门类型
     * @return java.math.BigDecimal
     * @author ZYJ
     * @date 2022/7/11 15:55
     */
    BigDecimal sumDeptPayOrderPrice(Long billId, Long deptId, Integer type);
}
