package com.yida.data.system.dto;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotBlank;
import java.io.Serializable;

/**
 * 代理商保存学校请求类
 *
 * @author ZYJ
 * @date 2021/9/27
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
@ApiModel(value = "AgentSchoolSaveDTO", description = "代理商保存学校请求类")
public class AgentSchoolSaveDTO implements Serializable {

    private static final long serialVersionUID = -982370233150683334L;

    /**
     * 学校主键id
     */
    @ApiModelProperty(value = "学校主键id")
    private Long deptId;

    /**
     * 学校名称
     */
    @NotBlank
    @ApiModelProperty(value = "学校名称", required = true)
    private String deptName;

    /**
     * 学校简称
     */
    @NotBlank
    @ApiModelProperty(value = "学校简称", required = true)
    private String deptAbbreviation;

    /**
     * 学校LOGO
     */
    @NotBlank
    @ApiModelProperty(value = "学校LOGO", required = true)
    private String icon;

    /**
     * 区域id
     */
    @ApiModelProperty(value = "区域id", required = true)
    private Long areaId;

    /**
     * 易达appId
     */
    @ApiModelProperty("易达appId")
    private Long yidaAppId;

    @ApiModelProperty("学校关联类型类型，0-企业微信，1-易达app，2-大华平台  多类型使用','连接")
    private String schoolType;

    /**
     * 微信corpId
     */
    @ApiModelProperty(value = "微信corpId")
    private String corpId;

    /**
     * 微信回调token，每个学校唯一确定
     */
    @ApiModelProperty(value = "微信回调token")
    private String wxCallToken;

    /**
     * 微信回调编码aes key，每个学校唯一确定
     */
    @ApiModelProperty(value = "微信回调编码aesKey")
    private String wxAesKey;

    /**
     * 微信公众平台id
     */
    @ApiModelProperty(value = "微信公众平台id")
    private String wxPublicId;

    /**
     * 微信支付api v3 key
     */
    @ApiModelProperty(value = "微信支付api v3 key")
    private String apiV3Key;

    /**
     * 商户号
     */
    @ApiModelProperty(value = "商户号")
    private String mchId;

    /**
     * api证书序列号
     */
    @ApiModelProperty(value = "api证书序列号")
    private String apiCreditSerial;

    /**
     * api证书私钥
     */
    @ApiModelProperty(value = "api证书私钥")
    private String apiCreditPrivateKey;

    /**
     * api证书私钥  无头尾
     */
    @ApiModelProperty(value = "api证书私钥  无头尾")
    private String apiCreditPrivateKeyNoHead;

    /**
     * 微信公众号appId
     */
    @ApiModelProperty("微信公众号appId")
    private String wxPublicAppId;

    /**
     * 微信公众号secret
     */
    @ApiModelProperty("微信公众号secret")
    private String wxPublicSecret;
}