package com.yida.data.system.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yida.data.common.core.entity.CurrentUser;
import com.yida.data.common.core.entity.system.Dept;
import com.yida.data.system.dto.AgentSchoolSelectPageDTO;
import com.yida.data.system.vo.AgentSchoolSelectPageVO;
import com.yida.data.system.vo.SchoolClassInfoVO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author MrBird
 */
public interface DeptMapper extends BaseMapper<Dept> {

    List<Dept> findListByAera(@Param("type") Integer type,
                              @Param("areaIds") List<Long> areaIds,
                              @Param("schoolName") String schoolName);

    SchoolClassInfoVO getSchoolAreaInfo(Long schoolId);

    /**
     * 查询代理商管理的学校列表数据
     *
     * @param toPage                   分页数据
     * @param agentSchoolSelectPageDTO 查询代理商管理的学校列表数据请求类
     * @param childIdList              子部门id集合
     * @param currentUser              当前登陆用户数据
     * @return com.baomidou.mybatisplus.core.metadata.IPage<com.yida.data.system.vo.AgentSchoolVO>
     * @author ZYJ
     * @date 2021/9/27 17:18
     */
    IPage<AgentSchoolSelectPageVO> listAgentSchoolPage(Page<Dept> toPage,
                                                       @Param("dto") AgentSchoolSelectPageDTO agentSchoolSelectPageDTO,
                                                       @Param("childIdList") List<Long> childIdList,
                                                       @Param("currentUser") CurrentUser currentUser);

    /**
     * 查询代理商关联学校
     *
     * @param agentId 代理商id
     * @param areaIds 区域id集合
     * @return java.util.List<com.yida.data.common.core.entity.system.Dept>
     * @author ZYJ
     * @date 2021/9/28 15:30
     */
    List<Dept> listSchoolByAgentId(@Param("agentId") Long agentId,
                                   @Param("areaIds") List<Long> areaIds,
                                   @Param("schoolName") String schoolName);

    /**
     * 具有创建账号的数据   自己创建的账号+当前登陆账号部门以下的账号
     *
     * @param currentUser 登陆用户
     * @param childIdList 子部门id集合
     * @param areaIds     区域id集合
     * @param schoolName  学校名称
     * @return java.util.List<com.yida.data.common.core.entity.system.Dept>
     * @author ZYJ
     * @date 2022/1/12 17:11
     */
    List<Dept> listSchool(@Param("currentUser") CurrentUser currentUser,
                          @Param("childIdList") List<Long> childIdList,
                          @Param("areaIds") List<Long> areaIds,
                          @Param("schoolName") String schoolName);
}