package com.yida.data.system.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.yida.data.common.core.entity.router.VueRouter;
import com.yida.data.common.core.entity.system.Menu;

import java.util.List;
import java.util.Map;

/**
 * @author MrBird
 */
public interface IMenuService extends IService<Menu> {

    /**
     * 获取登录用户角色原始菜单数据
     *
     * @param username 用户名
     * @return 用户权限
     */
    String findUserPermissions(String username);

    /**
     * 查询用户真实权限数据
     *
     * @param username   用户名
     * @param selectType 查询类型 0菜单,1按钮,空查询所有
     * @return 用户菜单
     */
    List<Menu> findUserRealMenus(String username, Integer selectType);

    /**
     * 获取用户菜单
     *
     * @param menu menu
     * @return 用户菜单
     */
    Map<String, Object> findMenus(Menu menu);

    /**
     * 获取用户路由
     *
     * @param username 用户名
     * @return 用户路由
     */
    List<VueRouter<Menu>> getUserRouters(String username);

    /**
     * 获取菜单列表
     *
     * @param menu menu
     * @return 菜单列表
     */
    List<Menu> findMenuList(Menu menu);

    /**
     * 创建菜单
     *
     * @param menu menu
     */
    void createMenu(Menu menu);

    /**
     * 更新菜单
     *
     * @param menu menu
     */
    void updateMenu(Menu menu);

    /**
     * 递归删除菜单/按钮
     *
     * @param menuIds menuIds
     */
    void deleteMeuns(String[] menuIds);

    /**
     * 根据登录账号查询菜单数 角色相关菜单数据
     *
     * @param username 登录账号
     * @return java.util.Map<java.lang.String, java.lang.Object>
     * @author ZYJ
     * @date 2022/4/6 13:50
     */
    Map<String, Object> listMenuByUserNameNoCheck(String username);
}
